package com.web.api.config;

import com.alibaba.fastjson.JSONException;
import com.web.api.dao.AnswerDao;
import com.web.api.exception.ErrorCode;
import com.web.api.exception.SgException;
import com.web.api.model.dto.AnswerDto;
import com.web.api.model.po.UserInfo;
import com.web.api.service.mail.MailServiceImpl;
import com.web.api.utils.ContextUtils;
import com.web.api.utils.DateUtills;
import lombok.extern.slf4j.Slf4j;
import org.springframework.aop.interceptor.AsyncUncaughtExceptionHandler;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Configuration;
import org.springframework.scheduling.annotation.AsyncConfigurer;
import org.springframework.util.ObjectUtils;
import javax.annotation.Resource;
import java.io.IOException;
import java.lang.reflect.Method;
import java.text.DecimalFormat;
import java.time.LocalDateTime;
import java.time.ZoneOffset;
import java.time.format.DateTimeFormatter;


/**
 * @author 王玉鹏
 * @version 1.0
 * @className AsyncExceptionConfig
 * @description TODO
 * @date 2020/7/12 21:42
 */
@Configuration
@Slf4j
public class AsyncExceptionConfig implements AsyncConfigurer {


    @Resource
    private AnswerDao answerDao;

    @Autowired
    private MailServiceImpl mailService;



    @Override
    public AsyncUncaughtExceptionHandler getAsyncUncaughtExceptionHandler() {
        return new SpringAsyncExceptionHandler();
    }

    class SpringAsyncExceptionHandler implements AsyncUncaughtExceptionHandler {
        @Override
        public void handleUncaughtException(Throwable throwable, Method method, Object... objects) {
            if (!(throwable instanceof SgException) &&
                    !(throwable instanceof IOException) &&
                    !(throwable instanceof NullPointerException) &&
                    !(throwable instanceof JSONException)){
                log.error("------非业务类异常，此处不做处理--------：{}", throwable);
                return;
            }
            log.info("业务参数：{}",objects);
            //收件人
            String[] toUser = {"251664727@qq.com", "632062365@qq.com"};
            AnswerDto answerDto = (AnswerDto) objects[2];
            Double startScore = (Double) objects[3];
            UserInfo userInfo = ContextUtils.getUserInfo(answerDto.getJobNo());
            DateTimeFormatter fmt = DateTimeFormatter.ofPattern("yyyy年MM月dd日 HH时mm分");
            userInfo.setEndTime(LocalDateTime.now().format(fmt));
            userInfo.setAnswerStatus("已结束");
            DecimalFormat df = new DecimalFormat("######0.00");
            Object addScores = ObjectUtils.isEmpty(userInfo.getFinalScore()) ? "本次答题异常终止，无法计算分数" : df.format(userInfo.getFinalScore() - startScore);
            if (answerDto.getBatchFlag()){
                ContextUtils.setBatchUserList(answerDto.getJobNo(),userInfo);
            }
            //关闭当前答题
            ContextUtils.setOnOff(answerDto.getJobNo(), false);
            String subject="";
            if (throwable instanceof SgException ) {
                subject=throwable.getMessage();
//                if (((SgException) throwable).getCode() == ErrorCode.LOGOUT.getCode()) {
//                    log.error("cookie已失效，移除本地用户：{}cookie", answerDto);
//                }
                if (((SgException) throwable).getCode() != ErrorCode.STOP_ANSWER.getCode()) {
                    userInfo.setErrMsg(throwable.getMessage());
                }
            }else {
                userInfo.setErrMsg("本次答题异常终止");
                subject=userInfo.getUserName()+"【"+userInfo.getJobNo()+"】答题异常结束";
            }
            int num = 0;
            if (!addScores.toString().equals("0.00")) {
                userInfo.setAddScores(addScores.toString());
                log.info("本次答题已结束，更新数据记录对应字段：{}", userInfo);
                num = answerDao.updatePassRecord(userInfo);
            } else {
                answerDao.deletePassRecord(userInfo.getId());
                log.error("本次答题增长积分小于等于0：{}，不保存此数据", addScores);
            }
            double startTime = LocalDateTime.parse(userInfo.getStartTime(),fmt).toEpochSecond(ZoneOffset.of("+8"));
            double endTime = LocalDateTime.parse(userInfo.getEndTime(),fmt).toEpochSecond(ZoneOffset.of("+8"));
            String elapsedTime =DateUtills.getElapsedTime(startTime,endTime);
            log.info("本次答题耗时：{}",elapsedTime);
            String loginFrom =  answerDto.getBatchFlag()?"批量导入答题":"用户登录";
            String content = "<html>\n" +
                    "<body>\n" +
                    "    <h3>用户：" + userInfo.getUserName() + " 已完成本次刷题，详情如下 ：</h3>\n" +
                    "    <p>工号：" + userInfo.getJobNo() + "</p>" +
                    "    <p>登录方式：" +loginFrom  + "</p>" +
                    "    <p>答题工种：<span style=\"color: blue;font-weight: bold\">" + answerDto.getJobTypeName() + "</span></p>" +
                    "    <p>初始分数：" + startScore + "</p>" +
                    "    <p>目标分数：" + userInfo.getTargetScore() + "</p>" +
                    "    <p>增长分数：<span style=\"color: blue;font-weight: bold\">" + addScores + "</span></p>" +
                    "    <p>最终分数：<span style=\"color: red;font-weight: bold\">" + userInfo.getFinalScore() + "</span></p>" +
                    "    <p>开始时间：" + userInfo.getStartTime() + "</p>" +
                    "    <p>结束时间：" + userInfo.getEndTime() + "</p>" +
                    "    <p>运行时间：<span style=\"color: blue;font-weight: bold\">" + elapsedTime + "</span></p>" +
                    "    <p>结束原因：<span style=\"color: red;font-weight: bold\">"  + throwable.getMessage() + "</span></p>" +
                    "    <a href=\"http://49.232.153.218/answer\">更多详情请前往系统查看!</a>" +
                    "</body>\n" +
                    "</html>";
            try {
                if (num > 0) {
                    if (ContextUtils.getBatchUserList().size()>10){
                        toUser = new String[]{"251664727@qq.com"};
                    }
                    mailService.sendSimpleMail(toUser, subject, content);
                } else {
                    log.error("答题结算更新失败：{}", userInfo);
                }
            } catch (Exception e) {
                log.error("邮件发送异常：{}", e);
            }
            //手动停止，不删除当前对象
            if (throwable instanceof SgException && ((SgException) throwable).getCode() == ErrorCode.STOP_ANSWER.getCode()) {
                return;
            }
            //删除当前答题对象
            ContextUtils.removeUserCookies(answerDto.getJobNo());
            ContextUtils.removeUserInfo(answerDto.getJobNo());
            log.error("------Async无返回方法的异常处理方法--------：{}", throwable);
        }
    }
}