package com.web.api.job;

import com.alibaba.excel.EasyExcel;
import com.web.api.exception.BusinessException;
import com.web.api.listener.ImportExcelListener;
import com.web.api.model.dto.AnswerDto;
import com.web.api.model.po.AnswerExcel;
import com.web.api.model.po.JobTypeEntity;
import com.web.api.service.AnswerJobService;
import com.web.api.service.AnswerService;
import com.web.api.service.mail.MailServiceImpl;
import com.web.api.utils.Constant;
import com.web.api.utils.ContextUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.http.client.CookieStore;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import org.springframework.util.ResourceUtils;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.InputStream;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author 王玉鹏
 * @version 1.0
 * @className sgAnswerScheduled
 * @description 定时任务
 * @date 2022/7/14 17:57
 */
@Slf4j
@Component
public class SgAnswerTask {


    @Autowired
    private AnswerService answerService;

    @Autowired
    private MailServiceImpl mailService;

    @Autowired
    private AnswerJobService answerJobService;


    /**
     * @return 每2小时执行一次封号操作
     */
//    @Scheduled(cron = "0 0 */2 * * ?")
    public void blockedAccount() {
        log.info("触发自动冻结账户定时任务...");
        List<Map<String, Object>> list = null;
        try {
            list = answerService.queryBlacklist();
        } catch (Exception e) {
            log.error("查询黑名单失败:", e);
        }
        List<Map<String, Object>> blacklist = list.stream().filter(e -> e.get("isOpen").equals(1)).collect(Collectors.toList());
        log.info("当前黑名单用户数量：{}",blacklist.size());
        blacklist.forEach(user -> {
            try {
                log.info("user=>{}",user);
                answerService.getForbidden(user.get("jobNo").toString());
                answerService.updateForbiddenStatus(user.get("jobNo").toString(), false,2);
                log.info("冻结【{}】账号成功",user.get("jobNo").toString());
            } catch (Exception e) {
                log.error("自动冻结账户任务异常:", e);
            }
        });
    }

    /**
     * 每天凌晨更新答题时间为无限
     */
    @Scheduled(cron = "0 0 0 * * ?")
    public void updateUnlimitedTimeTask() {
        log.info("定时更新答题时间上限为无限制...");
        try {
            answerService.clearAdminCookie();
            answerService.updateTime(0);
        } catch (Exception e) {
            log.error("执行定时任务异常:", e);
        }
    }

    /**
     * 每天上午6点更新答题时间上限为3小时
     */
    @Scheduled(cron = "0 0 6 * * ?")
    public void updateLimitTimeTask() {
        log.info("定时更新答题时间上限为3小时...");
        try {
            answerService.updateTime(3);
        } catch (Exception e) {
            log.error("执行定时任务异常:", e);
        }
    }
    /**
     * 每天凌晨01分批量导入excel答题
     */
    @Scheduled(cron = "0 1 0 * * ?")
    public void autoSgAnswer() {
        log.info("定时批量自动答题开启...");
        try {
//            File file = ResourceUtils.getFile("E:\\Documents\\WeChat Files\\wxid_3058230580912\\FileStorage\\MsgAttach\\642f0149642b0db6781c725100a51af9\\File\\2022-07\\bnsf.xlsx");
            File file = ResourceUtils.getFile("/opt/excel/bnsf.xlsx");
            InputStream inputStream = new FileInputStream(file);
            if (file == null) {
                log.info("发生错误{}", "文件为空");
                throw new BusinessException("文件为空");
            }
            if (!file.getName().endsWith(".xls") && !file.getName().endsWith(".xlsx")) {
                throw new BusinessException("文件格式不正确，只支持xls或xlsx的文件");
            }
            //每次导入批量答题前，清空批量用户列表
            ContextUtils.clearBatchUserList();
            log.info("文件格式校验通过，开始调用第三方接口答题");
            EasyExcel.read(inputStream, AnswerExcel.class, new ImportExcelListener(answerService, mailService)).sheet().doRead();
        } catch (FileNotFoundException e) {
            log.error("定时批量自动答题发生异常:", e);
        }
    }

    private void autoSgAnswer11(){
        //登录答题系统
        CookieStore cookieStore = null;
        try {
            cookieStore = answerService.login("W20091", "pengyu0215", "1111");
        } catch (Exception e) {
            log.error("执行自动答题任务异常:", e);
        }
        //根据工种名称获取工种基本信息拿到工种id和关卡初始id
        JobTypeEntity jobTypeEntity = Constant.jobTypeMap.get("电工");

        AnswerDto answerDto = new AnswerDto();
        answerDto.setJobNo("W20091");
        answerDto.setJobTypeName("电工");
        answerDto.setTargetScore(100000);
        answerDto.setJobTypeId(jobTypeEntity.getTypeId());
        answerDto.setJobGkId(jobTypeEntity.getGkId());
        answerDto.setBatchFlag(true);
        answerDto.setLevel(3);
        //开始答题
        try {
            answerService.answer(cookieStore, answerDto);
        } catch (Exception e) {
            log.error("执行定时任务异常:", e);
        }
    }

}
