package com.web.api.model;

import com.alibaba.fastjson.annotation.JSONField;
import lombok.Data;

import java.util.Date;
import java.util.function.Function;

/**
 * @author 王玉鹏
 * @version 1.0
 * @className DataResult
 * @description TODO
 * @date 2019/12/23 14:50
 */
@Data
public class DataResult <T > {
    /**
     * 状态
     **/
    @JSONField
    private int status;

    /**
     * 内容
     **/
    @JSONField
    private String message;

    /**
     * 内容
     **/
    @JSONField
    private T data;

    /**
     * 时间戳
     **/
    @JSONField
    private Date timestamp = new Date();

    public DataResult() {

    }
    private DataResult(int status, String message, T data) {
        this.status = status;
        this.message = message;
        this.data = data;
    }

    public static <V > DataResult<V> fail(int status, String message, V result) {
        return new DataResult(status, message, result);
    }

    public static <V> DataResult<V> fail(String message) {
        DataResult<V> result = new DataResult();
        result.setMessage(message);
        result.setStatus(-1);
        return result;
    }

    public static <V > DataResult<V> fail(int status, String message) {
        DataResult<V> result = new DataResult();
        result.setStatus(status);
        result.setMessage(message);
        return result;
    }

    public static <V > DataResult<V> ok() {
        DataResult<V> result = new DataResult();
        result.setMessage("success");
        return result;
    }

    public static <V > DataResult<V> ok(V data) {
        DataResult<V> result = new DataResult();
        result.setStatus(0);
        result.setMessage("success");
        result.setData(data);
        return result;
    }

    public static <V > DataResult<V> ok(String message, V data) {
        DataResult<V> result = new DataResult();
        result.setMessage(message);
        result.setData(data);
        return result;
    }

    public static <V > DataResult<V> ok(Integer status, String message, V data) {
        DataResult<V> result = new DataResult();
        result.setStatus(status);
        result.setMessage(message);
        result.setData(data);
        return result;
    }

    @Override
    public String toString() {
        return "DataResult{status =" + this.status + ", message =" + this.message + ", data =" + this.data + ", time = " + this.timestamp + "}";
    }

    public String toString(Function<DataResult<T>, String> fun) {
        return null == fun ? this.toString() : (String) fun.apply(this);
    }
}