package com.web.api.utils;

import io.jsonwebtoken.Claims;
import io.jsonwebtoken.Jwts;
import io.jsonwebtoken.SignatureAlgorithm;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * <pre>
 * 名称: JWT工具类
 * 描述:
 * </pre>
 *
 * @author SunGao
 * @date: 2019/2/21
 * @since v1.0.0
 */

public class JwtTokenUtils {

    private static final String CLAIM_KEY_USERNAME = "sub";
    private static final String CLAIM_KEY_CREATED = "created";
    private static final String CLAIM_KEY_USER_TYPE = "userType";
    public static final String JWT_SECRET = "dmzdmzdmzdmzdmzdmzdmzdmz";
    public static final int JWT_EXPIRATION = 3600;

    /**
     * 生成token
     *
     * @param
     * @return java.lang.String
     **/
    public static String generateToken(String userType,String username) {
        Map<String, Object> claims = new HashMap<>();
        claims.put(CLAIM_KEY_USERNAME, username);
        claims.put(CLAIM_KEY_CREATED, new Date());
        claims.put(CLAIM_KEY_USER_TYPE, userType);
        return generateToken(claims);
    }

    public static String generateToken(Map<String, Object> claims) {
        return Jwts.builder().setClaims(claims).setExpiration(generateExpirationDate())
                .signWith(SignatureAlgorithm.HS512, JWT_SECRET)
                .compact();
    }

    /**
     * 刷新token
     *
     * @param token
     * @return java.lang.String
     **/
    public static String refreshToken(String token) {
        String refreshedToken;
        try {
            Claims claims = getClaimsFromToken(token);
            claims.put(CLAIM_KEY_CREATED, new Date());
            refreshedToken = generateToken(claims);
        } catch (Exception e) {
            refreshedToken = null;
        }
        return refreshedToken;
    }


    /**
     * 从token获取用户名
     *
     * @param token
     * @return java.lang.String
     **/
    public static String getUsernameFromToken(String token) {
        String username;
        try {
            Claims claims = getClaimsFromToken(token);
            username = claims.getSubject();
        } catch (Exception e) {
            username = null;
        }
        return username;
    }

    /**
     * token是否过期时间
     *
     * @param token
     * @return java.lang.String
     **/
    public static Boolean isTokenExpired(String token) {
        Date expiration = getExpirationDateFromToken(token);
        return expiration.after(new Date());
    }


    /**
     * 生成token过期时间
     *
     * @param
     * @return java.lang.String
     **/
    public static Date generateExpirationDate() {
        return new Date(System.currentTimeMillis() + JWT_EXPIRATION * 1000);
    }

    /**
     * 从token获取过期时间
     *
     * @param token
     * @return java.lang.String
     **/
    public static Date getExpirationDateFromToken(String token) {
        Date expiration;
        try {
            Claims claims = getClaimsFromToken(token);
            expiration = claims.getExpiration();
        } catch (Exception e) {
            expiration = null;
        }
        return expiration;
    }

    public static Claims getClaimsFromToken(String token) {
        Claims claims;
        try {
            claims = Jwts.parser()
                    .setSigningKey(JWT_SECRET)
                    .parseClaimsJws(token)
                    .getBody();
        } catch (Exception e) {
            claims = null;
        }
        return claims;
    }

    /**
     * 验证token
     *
     * @param token, userDetails
     * @return java.lang.Boolean
     **/
    public static  Boolean validateToken(String token, String currentUserName) {
        String username = getUsernameFromToken(token);
        return username.equals(currentUserName) && isTokenExpired(token);
    }


}

